"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
Object.defineProperty(exports, "__esModule", { value: true });
exports.configureProgramForMultiEmit = void 0;
const node_core_library_1 = require("@rushstack/node-core-library");
// symbols for attaching hidden metadata to ts.Program instances.
const INNER_GET_COMPILER_OPTIONS_SYMBOL = Symbol('getCompilerOptions');
const INNER_EMIT_SYMBOL = Symbol('emit');
const JS_EXTENSION_REGEX = /\.js(\.map)?$/;
function wrapWriteFile(baseWriteFile, jsExtensionOverride) {
    if (!jsExtensionOverride) {
        return baseWriteFile;
    }
    const replacementExtension = `${jsExtensionOverride}$1`;
    return (fileName, data, writeBOM, onError, sourceFiles) => {
        return baseWriteFile(fileName.replace(JS_EXTENSION_REGEX, replacementExtension), data, writeBOM, onError, sourceFiles);
    };
}
function configureProgramForMultiEmit(innerProgram, ts, moduleKindsToEmit, mode) {
    const program = innerProgram;
    // Check to see if this Program has already been modified.
    let { [INNER_EMIT_SYMBOL]: innerEmit, [INNER_GET_COMPILER_OPTIONS_SYMBOL]: innerGetCompilerOptions } = program;
    if (!innerGetCompilerOptions) {
        program[INNER_GET_COMPILER_OPTIONS_SYMBOL] = innerGetCompilerOptions = program.getCompilerOptions;
    }
    if (!innerEmit) {
        program[INNER_EMIT_SYMBOL] = innerEmit = program.emit;
    }
    let foundPrimary = false;
    let defaultModuleKind;
    const multiEmitMap = new Map();
    for (const moduleKindToEmit of moduleKindsToEmit) {
        const kindCompilerOptions = moduleKindToEmit.isPrimary
            ? Object.assign({}, innerGetCompilerOptions()) : Object.assign(Object.assign({}, innerGetCompilerOptions()), { module: moduleKindToEmit.moduleKind, outDir: moduleKindToEmit.outFolderPath, 
            // Don't emit declarations for secondary module kinds
            declaration: false, declarationMap: false });
        if (!kindCompilerOptions.outDir) {
            throw new node_core_library_1.InternalError('Expected compilerOptions.outDir to be assigned');
        }
        if (mode === 'transpile') {
            kindCompilerOptions.declaration = false;
            kindCompilerOptions.declarationMap = false;
        }
        else if (mode === 'declaration') {
            kindCompilerOptions.emitDeclarationOnly = true;
        }
        if (moduleKindToEmit.isPrimary || mode !== 'declaration') {
            multiEmitMap.set(moduleKindToEmit, kindCompilerOptions);
        }
        if (moduleKindToEmit.isPrimary) {
            if (foundPrimary) {
                throw new Error('Multiple primary module emit kinds encountered.');
            }
            else {
                foundPrimary = true;
            }
            defaultModuleKind = moduleKindToEmit.moduleKind;
        }
    }
    const changedFiles = new Set();
    program.emit = (targetSourceFile, writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers, forceDtsEmit) => {
        if (emitOnlyDtsFiles) {
            return program[INNER_EMIT_SYMBOL](targetSourceFile, writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers, forceDtsEmit);
        }
        if (targetSourceFile && changedFiles) {
            changedFiles.add(targetSourceFile);
        }
        const originalCompilerOptions = program[INNER_GET_COMPILER_OPTIONS_SYMBOL]();
        let defaultModuleKindResult;
        const diagnostics = [];
        let emitSkipped = false;
        try {
            for (const [moduleKindToEmit, kindCompilerOptions] of multiEmitMap) {
                program.getCompilerOptions = () => kindCompilerOptions;
                // Need to mutate the compiler options for the `module` field specifically, because emitWorker() captures
                // options in the closure and passes it to `ts.getTransformers()`
                originalCompilerOptions.module = moduleKindToEmit.moduleKind;
                const flavorResult = program[INNER_EMIT_SYMBOL](targetSourceFile, writeFile && wrapWriteFile(writeFile, moduleKindToEmit.jsExtensionOverride), cancellationToken, emitOnlyDtsFiles, customTransformers, forceDtsEmit);
                emitSkipped = emitSkipped || flavorResult.emitSkipped;
                // Need to aggregate diagnostics because some are impacted by the target module type
                for (const diagnostic of flavorResult.diagnostics) {
                    diagnostics.push(diagnostic);
                }
                if (moduleKindToEmit.moduleKind === defaultModuleKind) {
                    defaultModuleKindResult = flavorResult;
                }
            }
            const mergedDiagnostics = ts.sortAndDeduplicateDiagnostics(diagnostics);
            return Object.assign(Object.assign({}, defaultModuleKindResult), { changedSourceFiles: changedFiles, diagnostics: mergedDiagnostics, emitSkipped });
        }
        finally {
            // Restore the original compiler options and module kind for future calls
            program.getCompilerOptions = program[INNER_GET_COMPILER_OPTIONS_SYMBOL];
            originalCompilerOptions.module = defaultModuleKind;
        }
    };
    return { changedFiles };
}
exports.configureProgramForMultiEmit = configureProgramForMultiEmit;
//# sourceMappingURL=configureProgramForMultiEmit.js.map